@@ -10,6 +10,7 @@ Library routines
     gexp
     glog
     mseq
+    acorr
 
 Adaptive cepstrum analysis
 --------------------------
@@ -48,6 +49,7 @@ LPC, LSP and PARCOR conversions
     lpc2lsp
     lpc2par
     par2lpc
+    lsp2lpc
     lsp2sp
 
 Mel-generalized cepstrum conversions
@@ -58,6 +60,7 @@ Mel-generalized cepstrum conversions
     mc2b
     b2mc
     c2acr
+    levdur
     c2ir
     ic2ir
     c2ndps
@@ -102,19 +105,26 @@ Waveform generation filters
 .. autosummary::
     :toctree: generated/
 
+    zerodf
+    zerodft
     poledf
+    poledft
     lmadf
     lspdf
     ltcdf
     glsadf
+    glsadft
     mlsadf
+    mlsadft
     mglsadf
+    mglsadft
 
 Utilities for waveform generation filters
 -----------------------------------------
 .. autosummary::
     :toctree: generated/
 
+    zerodf_delay
     poledf_delay
     lmadf_delay
     lspdf_delay
@@ -123,6 +133,13 @@ Utilities for waveform generation filters
     mlsadf_delay
     mglsadf_delay
 
+Metrics
+-------
+.. autosummary::
+    :toctree: generated/
+
+    cdist
+
 """
 
 import numpy as np
@@ -130,6 +147,51 @@ import numpy as np
 from . import _sptk
 from pysptk.util import apply_along_last_axis, automatic_type_conversion
 
+
+def cdist(c1, c2, otype=0, frame=False):
+    """Calculation of cepstral distance
+
+    Parameters
+    ----------
+    c1 : array
+        Minimum-phase cepstrum
+    c2 : array
+        Minimum-phase cepstrum
+    otype : int
+        Output data type
+            (0) [db]
+            (1) squared error
+            (2) root squared error
+
+        Default is 0.
+    frame : bool
+        If True, returns frame-wise distance, otherwise returns mean distance.
+        Default is False.
+
+    Returns
+    -------
+    distance
+
+    """
+
+    if otype not in [0, 1, 2]:
+        raise ValueError("unsupported otype: %d, must be in 0:2" % otype)
+
+    assert c1.shape[0] == c2.shape[0]
+    T = c1.shape[0]
+
+    s = ((c1[:, 1:] - c2[:, 1:])**2).sum(-1)
+
+    if otype == 0:
+        s = np.sqrt(2 * s) * 10 / np.log(10)
+    elif otype == 2:
+        s = np.sqrt(s)
+
+    if frame:
+        return s
+    else:
+        return s.mean()
+
 ### Library routines ###
 
 
@@ -201,6 +263,32 @@ def mseq():
     return _sptk.mseq()
 
 
+@apply_along_last_axis
+@automatic_type_conversion
+def acorr(x, order):
+    """Autocorrelation
+
+    Parameters
+    ----------
+    x : array
+        Input frame
+
+    order : int
+        Order of sequence
+
+    Returns
+    -------
+    r : array
+        Autocorrelation
+
+    See Also
+    --------
+    pysptk.sptk.levdur
+    pysptk.sptk.c2acr
+    """
+    return _sptk.acorr(x, order)
+
+
 ### Adaptive mel-generalized cepstrum analysis ###
 
 def acep(x, c, lambda_coef=0.98, step=0.1, tau=0.9, pd=4, eps=1.0e-6):
@@ -758,7 +846,7 @@ def fftcep(logsp,
 
 @apply_along_last_axis
 @automatic_type_conversion
-def lpc(windowed, order=25, min_det=1.0e-6):
+def lpc(windowed, order=25, min_det=1.0e-6, use_scipy=True):
     """Linear prediction analysis
 
     Parameters
@@ -773,6 +861,10 @@ def lpc(windowed, order=25, min_det=1.0e-6):
         Mimimum value of the determinant of normal matrix.
         Default is 1.0e-6.
 
+    use_scipy : bool
+        Use scipy's solve_toeplitz implementation or not. Default is True.
+        This would be more numerically stable than SPTK.
+
     Returns
     -------
     a : array, shape (``order + 1``)
@@ -797,7 +889,11 @@ def lpc(windowed, order=25, min_det=1.0e-6):
     pysptk.sptk.lspdf
 
     """
-    return _sptk.lpc(windowed, order, min_det)
+    if use_scipy:
+        r = _sptk.acorr(windowed, order)
+        return levdur(r, use_scipy=True)
+    else:
+        return _sptk.lpc(windowed, order, min_det)
 
 
 ### MFCC ###
@@ -916,8 +1012,8 @@ def lpc2c(lpc, order=None):
 
 @apply_along_last_axis
 @automatic_type_conversion
-def lpc2lsp(lpc, numsp=512, maxiter=4, eps=1.0e-6, loggain=False, otype=0,
-            fs=None):
+def lpc2lsp(lpc, numsp=128, maxiter=4, eps=1.0e-6, has_gain=True,
+            loggain=False, otype=0, fs=None):
     """LPC to LSP
 
     Parameters
@@ -926,7 +1022,7 @@ def lpc2lsp(lpc, numsp=512, maxiter=4, eps=1.0e-6, loggain=False, otype=0,
         LPC
 
     numsp : int, optional
-        Number of unit circle. Default is 512.
+        Number of unit circle. Default is 128.
 
     maxiter : int, optional
         Maximum number of iteration. Default is 4.
@@ -934,6 +1030,9 @@ def lpc2lsp(lpc, numsp=512, maxiter=4, eps=1.0e-6, loggain=False, otype=0,
     eps : float, optional
         End condition for iteration. Default is 1.0e-6.
 
+    has_gain : bool, optional
+        Whether input LPC has gain at the index 0 or not. Default is True.
+
     loggain : bool, optional
         whether the converted lsp should have loggain or not.
         Default is False.
@@ -967,7 +1066,7 @@ def lpc2lsp(lpc, numsp=512, maxiter=4, eps=1.0e-6, loggain=False, otype=0,
 
     """
 
-    return _sptk.lpc2lsp(lpc, numsp, maxiter, eps, loggain, otype, fs)
+    return _sptk.lpc2lsp(lpc, numsp, maxiter, eps, has_gain, loggain, otype, fs)
 
 
 @apply_along_last_axis
@@ -1023,7 +1122,56 @@ def par2lpc(par):
 
 @apply_along_last_axis
 @automatic_type_conversion
-def lsp2sp(lsp, fftlen=256):
+def lsp2lpc(lsp, has_gain=True, loggain=False, fs=None, itype=0):
+    """LSP to LPC
+
+
+    Parameters
+    ----------
+    lpc : array
+        LPC
+
+    has_gain : bool, optional
+        Whether input LPC has gain at the index 0 or not. Default is True.
+
+    loggain : bool, optional
+        If True, it's assumed that input LPC has loggain and convert it to linear
+        gain. Default is False.
+
+    fs : int, optional
+        Sampling frequency. Default is None and unused.
+
+    itype : int, optional
+        Input LPC format
+            (0)  normalized frequency (0 ~ pi)
+            (1)  normalized frequency (0 ~ 0.5)
+            (2)  frequency (kHz)
+            (3)  frequency (Hz)
+
+        Default is 0.
+
+    Returns
+    -------
+    lsp : array, shape (``order + 1``) if has_gain else (``order``)
+        LPC
+
+    raises
+    ------
+    ValueError
+        if ``fs`` is not specified when itype = 2 or 3.
+        if ``loggain`` and not ``has_gain``.
+
+    See Also
+    --------
+    pysptk.sptk.lpc2lsp
+    """
+
+    return _sptk.lsp2lpc(lsp, has_gain, loggain, fs, itype)
+
+
+@apply_along_last_axis
+@automatic_type_conversion
+def lsp2sp(lsp, fftlen=256, has_gain=True, loggain=False, fs=None, itype=0):
     """LSP to spectrum
 
     Parameters
@@ -1034,17 +1182,30 @@ def lsp2sp(lsp, fftlen=256):
     fftlen : int, optional
         FFT length
 
-    TODO: consider ``otype`` optional argument
+    has_gain : bool, optional
+        Whether input LPC has gain at the index 0 or not. Default is True.
+
+    loggain : bool, optional
+        If True, it's assumed that input LPC has loggain and convert it to linear
+        gain. Default is False.
+
+    fs : int, optional
+        Sampling frequency. Default is None and unused.
+
+    itype : int, optional
+        Input LPC format
+            (0)  normalized frequency (0 ~ pi)
+            (1)  normalized frequency (0 ~ 0.5)
+            (2)  frequency (kHz)
+            (3)  frequency (Hz)
+
+        Default is 0.
 
     Returns
     -------
     sp : array, shape
         Spectrum. ln|H(z)|.
 
-    Notes
-    -----
-    It is asuumed that ``lsp`` has loggain at ``lsp[0]``.
-
     See Also
     --------
     pysptk.sptk.lpc2par
@@ -1154,6 +1315,7 @@ def c2acr(c, order=None, fftlen=256):
     pysptk.sptk.uels
     pysptk.sptk.c2ir
     pysptk.sptk.lpc2c
+    pysptk.sptk.levdur
 
     """
 
@@ -1507,7 +1669,10 @@ def mgc2sp(ceps, alpha=0.0, gamma=0.0, fftlen=256):
     Returns
     -------
     sp : array, shape (``fftlen // 2 + 1``)
-         Complex spectrum
+         Complex spectrum log_e(H(ω))
+
+    .. warning:: There's no equivalent option ``o`` in SPTK's mgc2sp.
+        You have to normalize spectrum manually if needed.
 
     Raises
     ------
@@ -1870,6 +2035,90 @@ def rectangular(n, normalize=1):
 
 ### Waveform generation filters ###
 
+def zerodf_delay(order):
+    """Delay for zerodf
+
+    Parameters
+    ----------
+    order : int
+        Order of zerodf filter coefficients
+
+    Returns
+    -------
+    delay : array
+        Delay
+
+    """
+    return np.zeros(_sptk.zerodf_delay_length(order))
+
+
+def zerodf(x, b, delay):
+    """All zero digital filter
+
+    Parameters
+    ----------
+    x : float
+        A input sample
+
+    b : array
+        FIR parameters
+
+    delay : array
+        Delay
+
+    Returns
+    -------
+    y : float
+        A filtered sample
+
+    Raises
+    ------
+    ValueError
+        if invalid delay length is supplied
+
+    See Also
+    --------
+    pysptk.sptk.lpc
+    pysptk.sptk.ltcdf
+    pysptk.sptk.lmadf
+
+    """
+
+    return _sptk.zerodf(x, b, delay)
+
+
+def zerodft(x, b, delay):
+    """Transpose All zero digital filter
+
+    Parameters
+    ----------
+    x : float
+        A input sample
+
+    b : array
+        FIR parameters
+
+    delay : array
+        Delay
+
+    Returns
+    -------
+    y : float
+        A filtered sample
+
+    Raises
+    ------
+    ValueError
+        if invalid delay length is supplied
+
+    See Also
+    --------
+    pysptk.sptk.zerodf
+
+    """
+
+    return _sptk.zerodft(x, b, delay)
+
 
 def poledf_delay(order):
     """Delay for poledf
@@ -1923,6 +2172,39 @@ def poledf(x, a, delay):
     return _sptk.poledf(x, a, delay)
 
 
+def poledft(x, a, delay):
+    """Transpose All-pole digital filter
+
+    Parameters
+    ----------
+    x : float
+        A input sample
+
+    a : array
+        AR coefficients
+
+    delay : array
+        Delay
+
+    Returns
+    -------
+    y : float
+        A filtered sample
+
+    Raises
+    ------
+    ValueError
+        if invalid delay length is supplied
+
+    See Also
+    --------
+    pysptk.sptk.poledf
+
+    """
+
+    return _sptk.poledft(x, a, delay)
+
+
 def lmadf_delay(order, pd):
     """Delay for lmadf
 
@@ -2157,6 +2439,44 @@ def glsadf(x, c,
     return _sptk.glsadf(x, c, stage, delay)
 
 
+def glsadft(x, c,
+            stage,
+            delay):
+    """Transpose GLSA digital filter
+
+    Parameters
+    ----------
+    x : float
+        A input sample
+
+    c : array
+        Geneeraized cepstrum
+
+    stage : int
+        -1 / gamma
+
+    delay : array
+        Delay
+
+    Returns
+    -------
+    y : float
+        A filtered sample
+
+    Raises
+    ------
+    ValueError
+        - if invalid number of stage is specified
+        - if invalid delay length is supplied
+
+    See Also
+    --------
+    pysptk.sptk.glsadf
+    """
+
+    return _sptk.glsadft(x, c, stage, delay)
+
+
 def mlsadf_delay(order, pd):
     """Delay for mlsadf
 
@@ -2225,6 +2545,46 @@ def mlsadf(x, b, alpha, pd, delay):
     return _sptk.mlsadf(x, b, alpha, pd, delay)
 
 
+def mlsadft(x, b, alpha, pd, delay):
+    """Transpose MLSA digital filter
+
+    Parameters
+    ----------
+    x : float
+        A input sample
+
+    b : array
+        MLSA filter coefficients
+
+    alpha : float
+        All-pass constant
+
+    pd : int
+        Order of pade approximation
+
+    delay : array
+        Delay
+
+    Returns
+    -------
+    y : float
+        A filtered sample
+
+    Raises
+    ------
+    ValueError
+        - if invalid order of pade approximation is specified
+        - if invalid delay length is supplied
+
+    See Also
+    --------
+    pysptk.sptk.mlsadf
+
+    """
+
+    return _sptk.mlsadft(x, b, alpha, pd, delay)
+
+
 def mglsadf_delay(order, stage):
     """Delay for mglsadf
 
@@ -2292,8 +2652,47 @@ def mglsadf(x, b, alpha, stage, delay):
     return _sptk.mglsadf(x, b, alpha, stage, delay)
 
 
+def mglsadft(x, b, alpha, stage, delay):
+    """Transpose MGLSA digital filter
+
+    Parameters
+    ----------
+    x : float
+        A input sample
+
+    b : array
+        MGLSA filter coefficients
+
+    alpha : float
+        All-pass constant
+
+    stage : int
+        -1 / gamma
+
+    delay : array
+        Delay
+
+    Returns
+    -------
+    y : float
+        A filtered sample
+
+    Raises
+    ------
+    ValueError
+        - if invalid number of stage is specified
+        - if invalid delay length is supplied
+
+    See Also
+    --------
+    pysptk.sptk.mglsadf
+    """
+
+    return _sptk.mglsadft(x, b, alpha, stage, delay)
+
 ### Excitation ###
 
+
 def excite(pitch, hopsize=100, interp_period=1, gaussian=False, seed=1):
     """Excitation generation
 
@@ -2346,3 +2745,47 @@ def phidf(x, order, alpha, delay):
 
 def lspcheck(lsp):
     return _sptk.lspcheck(lsp)
+
+
+@apply_along_last_axis
+@automatic_type_conversion
+def levdur(r, eps=0.0, use_scipy=True):
+    """Solve an Autocorrelation Normal Equation Using Levinson-Durbin Method
+
+    Parameters
+    ----------
+    r : array
+        Autocorrelation sequence
+
+    eps : float
+        Singular check (eps(if -1., 0.0 is assumed))
+
+    use_scipy : bool
+        Use scipy's solve_toeplitz implementation or not. Default is True.
+        This would be more numerically stable than SPTK.
+
+    Returns
+    -------
+    a : array
+        LPC
+
+    Raises
+    ------
+    RuntimeError
+        - if abnormally completed or unstable LPC
+
+    See Also
+    --------
+    pysptk.sptk.c2acr
+    """
+    if use_scipy:
+        from scipy.linalg import solve_toeplitz
+        a = np.empty_like(r)
+        # Rx = r where R is a toeplitz matrix
+        a[1:] = -solve_toeplitz(r[:-1], r[1:])
+        # http://www.seas.ucla.edu/~ingrid/ee213a/speech/vlad_present.pdf
+        G = np.sqrt(r[0] + (a[1:] * r[1:]).sum())
+        a[0] = G
+        return a
+    else:
+        return _sptk.levdur(r, eps)
